#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

//#define _GNU_SOURCE
//#include <getopt.h>

#include <sys/types.h>
#include <openssl/md5.h>


#define WEPKEYSIZE              5
#define WEPSTRONGKEYSIZE        13
#define WEPKEYS                 4
#define WEPKEYSTORE             (WEPKEYSIZE * WEPKEYS)

//Convert integer to character representing hex
#define HEX(a) ( (a)<10 ? (a) + '0' : (a) -10 + 'A' )



void wep_keygen128(const char *str, u_char *keys)
{
	MD5_CTX ctx;
	u_char buf[64];
	int i, j;

	
	//Fill 64B buffer with passphrase until buffer is full
	for(i = 0,j=0; i < 64; i++)
	{
		if(str[j] == 0)
			j = 0;
		buf[i] = str[j++];
	}

	//Make MD5 hash over buffer
	MD5_Init(&ctx);
	MD5_Update(&ctx, buf, sizeof(buf));
	MD5_Final(buf, &ctx);

	//Backcopy keys
	for(i = 0; i < WEPSTRONGKEYSIZE; i++)
		keys[i] = buf[i];

}

//Print key to stdout
//keys ... the key string
//cnt ... the amount of key digits
//separator the separator sign after 4 hex-digits
void printKeys(u_char* keys, int cnt, char separator)
{
	//Print it
	for(int l=0; l<cnt; l++)
	{
		printf("%c%c",  HEX((0x00f0 & keys[l])>>4), HEX(0x000f & keys[l]));

		//Print separator after 4 hex digits
		if( (l-1)%2==0 && l>0 && l<15 && separator)
			printf("%c", separator);
	}
}


//Print the usage of tool
void printUsage()
{
	printf("Create 104bit WEP-Key from a passphrase\n\n");
	printf("usage: WepPassphrase [-n] [-s] passphrase\n");
	printf("or:    WepPassphrase -h\n");
	printf("\n");
	printf("-n    Don't put newline at end\n");
	printf("-s    Don't print separating '-'\n");
	printf("-h    Print this help\n");
}


int main(int argc, char*argv[])
{
	//The key data
	u_char keys[WEPSTRONGKEYSIZE];

	//Option of parsing command line args
	int opt=0;

	//Indicate, if newline should be printed at end
	int printNewline=1;
	//The separator of 4 hex values
	char separator='-';

	
	//We need 2 arguments at least
	if(argc<2)
	{
		printf("Not enough arguments!\n");
		//Print the usage
		printUsage();
		exit(-1);
	}
	
	
	//Parse command options
	while((opt = getopt(argc, argv, "nsh")) != -1)
	{
		switch(opt)
		{
			//Don't print newline
			case 'n':
				printNewline=0;
				break;
			//Don't use separator
			case 's':
				separator=0;
				break;
			//Don't use separator
			case 'h':
				printUsage();
				exit(0);
				break;
			//Unknown parameter
			case '?':
				printUsage();
				exit(-1);
				break;
		}
	}

	
	//Check if last argument is passphrase
	if(argv[argc-1][0] == '-')
	{
		printf("Last argument has to be the passphrase!\n");
		printUsage();
		exit(-1);
	}
	
	
	//Calc key
	wep_keygen128(argv[argc-1], keys);
	//Print keys
	printKeys(keys, WEPSTRONGKEYSIZE, separator);


	//Check if we should print a newline
	if(printNewline)
		printf("\n");

	return 0;
}
